@@ -8,13 +8,15 @@
 #'   \code{TRUE}.
 #' @param ids logical. include a column for individual identifiers (\code{ids})?
 #' @param strata logical. include a column for current statification (\code{strata})?
-#' @param ... additional arguments ot be passed to or from methods.
+#' @param sort.alleles logical. for non-haploid objects, should alleles be sorted 
+#'   in genotypes or left as in original order? (only takes affect if \code{one.col = TRUE})
+#' @param ... additional arguments to be passed to or from methods.
 #'   
 #' @return A \code{matrix} with one row per sample.
 #' 
 #' @author Eric Archer \email{eric.archer@@noa.gov}
 #' 
-#' @seealso \link{df2gtypes} \link[strataG]{as.data.frame}
+#' @seealso \link{df2gtypes} \link[strataG]{as.data.frame} \link[strataG]{as.array}
 #' 
 #' @examples 
 #' data(msats.g)
@@ -31,40 +33,49 @@
 #' genotypes.mat <- as.matrix(msats.g, one.col = TRUE, ids = FALSE, strata = FALSE)
 #' head(genotypes.mat)
 #' 
-#' @importFrom methods setMethod
 #' @aliases as.matrix,gtypes-method as.matrix.gtypes as.matrix
+#' @importFrom methods setMethod
+#' @importFrom utils unstack
 #' 
 #' @export
 #' 
 setMethod(
   "as.matrix", "gtypes",
-  function(x, one.col = FALSE, sep = "/", ids = TRUE, strata = TRUE, ...) {
-    # create matrix identifying which rows (columns) each sample (row) is in 
-    #   in the @loci slot
-    id.mat <- matrix(1:nrow(x@loci), ncol = ploidy(x))
-    # loop through each locus
-    gen.mat <- do.call(cbind, lapply(locNames(x), function(locus) {
-      # extract alleles for this locus as a matrix with one allele per colum
-      this.loc <- x@loci[[locus]]
-      this.loc <- apply(id.mat, 1, function(i) as.character(this.loc[i]))
-      this.loc <- if(is.vector(this.loc)) cbind(this.loc) else t(this.loc)  
-      # collapse alleles to create a one column matrix if one.col == TRUE
-      if(one.col & x@ploidy > 1) {
-        this.loc <- cbind(apply(this.loc, 1, function(alleles) {
-          if(any(is.na(alleles))) NA else paste(alleles, collapse = sep)
-        }))
+  function(x, one.col = FALSE, sep = "/", ids = TRUE, 
+           strata = TRUE, sort.alleles = TRUE, ...) {
+    setkey(x@data, ids)
+    
+    gen.mat <- if(one.col) {
+      # one column per locus
+      .combineLoci <- function(x, sep, sort) {
+        x <- as.character(x)
+        if(any(is.na(x))) {
+          as.character(NA)
+        } else {
+          x <- if(sort) sort(x) else x
+          paste(x, collapse = sep)
+        }
       }
-      # assign column names
-      colnames(this.loc) <- if(ncol(this.loc) > 1) {
-        paste(locus, 1:ncol(this.loc), sep = ".")
+      mat <- x@data[, lapply(.SD, .combineLoci, sep = sep, sort = sort.alleles), 
+                    by = "ids", .SDcols = !c("ids", "strata")]
+      as.matrix(mat)[, -1]
+    } else {
+      # alleles on separate columns
+      dt <- x@data[, unlist(.SD), by = ids, .SDcols = !c("ids", "strata")] 
+      mat <- unstack(dt, form = V1 ~ ids)
+      pl <- ploidy(x)
+      colnames(mat) <- if(pl > 1) {
+        mat <- t(mat)
+        paste(rep(locNames(x), each = pl), 1:pl, sep = ".")
       } else {
-        locus
+        locNames(x)
       }
-      this.loc
-    }))
+      mat
+    }
+    # finish formatting matrix
     rownames(gen.mat) <- indNames(x)
     if(strata) gen.mat <- cbind(strata = as.character(strata(x)), gen.mat)
     if(ids) gen.mat <- cbind(ids = indNames(x), gen.mat)
-    gen.mat    
+    gen.mat
   }
 )
\ No newline at end of file
