@@ -6,13 +6,14 @@
 #'   individuals must share to be considered duplicate individuals.
 #' @param num.cores number of CPU cores to use.
 #' 
-#' @return a data.frame with the following columns:
+#' @return if no duplicates are present, the result is \code{NULL}, otherwise
+#'   a data.frame with the following columns is returned:
 #' \tabular{ll}{
 #'   \code{ids.1, ids.2} \tab sample ids.\cr
-#'   \code{strata1, strata2} \tab sample strata.\cr
+#'   \code{strata.1, strata.2} \tab sample stratification.\cr
 #'   \code{num.loci.genotyped} \tab number of loci genotyped for both 
 #'     samples.\cr
-#'   \code{num.loci.shared} \tab number of loci shared between both samples.\cr
+#'   \code{num.loci.shared} \tab number of loci shared (all alleles the same) between both samples.\cr
 #'   \code{prop.loci.shared} \tab proportion of loci genotyped for both samples 
 #'     that are shared.\cr
 #'   \code{mismatch.loci} \tab loci where the two samples do not match.\cr
@@ -36,8 +37,8 @@ dupGenotypes <- function(g, num.shared = 0.8, num.cores = 1) {
   shared.locs <- propSharedLoci(g, type = "ids", num.cores = num.cores)
   dup.df <- shared.locs[shared.locs[, "prop.same"] >= num.shared, ]
   if(nrow(dup.df) > 0) {
-    dup.df$strata.1 <- strata(g)[dup.df$ids.1]
-    dup.df$strata.2 <- strata(g)[dup.df$ids.2]
+    dup.df$strata.1 <- as.character(strata(g)[dup.df$ids.1])
+    dup.df$strata.2 <- as.character(strata(g)[dup.df$ids.2])
     dup.df$mismatch.loci <- sapply(1:nrow(dup.df), function(i) {
       shared.prop <- as.matrix(dup.df[i, locNames(g)])
       loc.diff <- locNames(g)[which(shared.prop < 1)]
@@ -59,5 +60,6 @@ dupGenotypes <- function(g, num.shared = 0.8, num.cores = 1) {
     rownames(dup.df) <- NULL
   } else dup.df <- NULL
   
+  if(is.null(dup.df)) cat("No duplicates found. NULL returned.\n")
   dup.df
 }
