@@ -1,8 +1,9 @@
 #' @title Find and label haplotypes
 #' @description Identify and group sequences that share the same haplotype.
 #'
-#' @param x a \code{\link{DNAbin}} \code{\link{multidna}}, or
-#'   \linkS4class{gtypes} object.
+#' @param x sequences in a \code{character matrix}, \code{list},
+#'   or \code{\link{DNAbin}} object, or a haploid \linkS4class{gtypes} object
+#'   with sequences.
 #' @param prefix a character string giving prefix to be applied to numbered
 #'   haplotypes. If NULL, haplotypes will be labeled with the first label
 #'   from original sequences.
@@ -18,22 +19,23 @@
 #'   \code{unassigned} element.
 #'
 #' @return
-#'  For \code{DNAbin} or \code{multidna}, a list with the following elements:
+#'  For \code{character}, \code{list}, or \code{DNAbin}, a list with the following elements:
 #'  \describe{
 #'    \item{haps}{named vector (\code{DNAbin}) or list of named vectors
 #'      (\code{multidina}) of haplotypes for each sequence in \code{x}.}
-#'    \item{hap.seqs}{\code{DNAbin} or \code{multidna} object containing 
+#'    \item{hap.seqs}{\code{DNAbin} or \code{multidna} object containing
 #'      sequences for each haplotype.}
-#'    \item{unassigned}{\code{data.frame} listing closest matching haplotypes 
-#'      and the number of substitutions different. Will be \code{NULL} if no 
-#'      sequences remain unassigned.}
+#'    \item{unassigned}{\code{data.frame} listing closest matching haplotypes
+#'      for unassignable sequences with N's and the minimum number of
+#'      substitutions between the two. Will be \code{NULL} if no sequences
+#'      remain unassigned.}
 #'  }
-#'  
+#'
 #'  For \code{gtypes}, a list with the following elements:
 #'  \describe{
 #'    \item{gtypes}{the new \code{gtypes} object with the haplotypes reassigned.}
-#'    \item{unassigned}{a list containing the \code{unassigned} attribute 
-#'      \code{data.frame} for each gene if present, otherwise \code{NULL}.}
+#'    \item{unassigned}{a list containing the \code{unassigned} \code{data.frame}
+#'      for each gene if present, otherwise \code{NULL}.}
 #'  }
 #'
 #' @author Eric Archer \email{eric.archer@@noaa.gov}
@@ -54,20 +56,17 @@
 #' sample.seqs <- lapply(sample.seqs, function(x) strsplit(haps[x], "")[[1]])
 #' names(sample.seqs) <- ids
 #'
-#' # add some random ambiguities
+#' # add 1-2 random ambiguities
 #' with.error <- sample(1:length(sample.seqs), 10)
 #' for(i in with.error) {
-#'   site <- sample(1:length(sample.seqs[[i]]), 1)
-#'   sample.seqs[[i]][site] <- "n"
+#'   num.errors <- sample(1:2, 1)
+#'   sites <- sample(1:length(sample.seqs[[i]]), num.errors)
+#'   sample.seqs[[i]][sites] <- "n"
 #' }
 #'
-#' # convert to DNAbin
-#' library(ape)
-#' sample.seqs <- as.DNAbin(sample.seqs)
-#'
 #' hap.assign <- labelHaplotypes(sample.seqs, prefix = "Hap.")
 #' hap.assign
-#' 
+#'
 #' @name labelHaplotypes
 #' @importFrom swfscMisc zero.pad
 #' @export
@@ -76,6 +75,7 @@ labelHaplotypes <- function(x, prefix = NULL, use.indels = TRUE) {
   UseMethod("labelHaplotypes")
 }
 
+
 #' @rdname labelHaplotypes
 #' @export
 #'
@@ -90,16 +90,23 @@ labelHaplotypes.default  <- function(x, prefix = NULL, use.indels = TRUE) {
     return(list(haps = haps, hap.seqs = x, unassigned = NULL))
   }
 
+  # throw error if any sequence names are duplicated
+  if(any(duplicated(rownames(x)))) {
+    warning("'x' cannot have duplicate sequence names. NULL returned.",
+            call. = FALSE, immediate. = TRUE)
+    return(NULL)
+  }
+
   # find sequences without Ns
   has.ns <- apply(as.character(x), 1, function(bases) "n" %in% tolower(bases))
-  if(sum(!has.ns) == 1) {
-    warning("There is only one sequence without ambiguities (N's). Can't assign haplotypes. NULL returned.",
+  if(sum(!has.ns) <= 1) {
+    warning("There less than two sequences without ambiguities (N's). Can't assign haplotypes. NULL returned.",
             call. = FALSE, immediate. = TRUE)
     return(NULL)
   }
 
   # get pairwise distances and set all non-0 distances to 1
-  x.no.ns <- x[!has.ns, ]
+  x.no.ns <- x[!has.ns, , drop = FALSE]
   hap.dist <- dist.dna(x.no.ns, model = "N", pairwise.deletion = TRUE)
   if(use.indels) hap.dist <- hap.dist + dist.dna(x.no.ns, model = "indelblock")
   hap.dist <- as.matrix(hap.dist)
@@ -129,56 +136,73 @@ labelHaplotypes.default  <- function(x, prefix = NULL, use.indels = TRUE) {
   hap.seqs <- x[names(unique.codes), , drop = FALSE]
   rownames(hap.seqs) <- unique.codes
   hap.seqs <- hap.seqs[order(rownames(hap.seqs)), , drop = FALSE]
+  hap.seqs <- as.character(as.matrix(hap.seqs))
 
-  unassigned.df <- if(!all(!has.ns)) {
-    # calculate distance between haplotypes and samples with Ns
-    with.ns.seqs <- x[has.ns, ]
-    haps.and.ns <- rbind(hap.seqs, with.ns.seqs)
-    hap.dist <- dist.dna(haps.and.ns, model = "N", pairwise.deletion = TRUE)
-    if(use.indels) hap.dist <- hap.dist + dist.dna(haps.and.ns, model = "indelblock")
-    hap.dist <- as.matrix(hap.dist)[rownames(with.ns.seqs), rownames(hap.seqs), drop = FALSE]
+  # get distance of all sequences with n's to other sequences (possible sequences)
+  x.has.ns <- as.character(x)[has.ns, , drop = FALSE]
+  unk.dist <- sapply(rownames(x.has.ns), function(i) {
+    # get sites with n's in this sequence
+    n.sites <- which(tolower(x.has.ns[i, ]) == "n")
+    if(length(n.sites) == 0) return(NULL)
+    # create matrix of sites that n's are at and remove sequences with n's
+    possible.sites <- unique(hap.seqs[, n.sites, drop = FALSE])
+    # create sequences to test
+    if(nrow(possible.sites) == 0) return(NULL)
+    test.seqs <- do.call(rbind, lapply(1:nrow(possible.sites), function(j) {
+      seq.j <- x.has.ns[i, ]
+      seq.j[n.sites] <- possible.sites[j]
+      seq.j
+    }))
+    test.names <- paste("test.seq.", 1:nrow(test.seqs), sep = "")
+    rownames(test.seqs) <- test.names
+    # get distances between test sequences and sequences without n's
+    new.mat <- as.DNAbin(rbind(hap.seqs, test.seqs))
+    hap.dist <- dist.dna(new.mat, model = "N", pairwise.deletion = TRUE, as.matrix = TRUE)
+    if(use.indels) hap.dist <- hap.dist + dist.dna(new.mat, model = "indelblock", as.matrix = TRUE)
+    hap.dist[test.names, !colnames(hap.dist) %in% test.names, drop = FALSE]
+  }, simplify = FALSE, USE.NAMES = TRUE)
 
-    # match samples with Ns to one or more haplotypes
-    ns.matches <- apply(hap.dist, 1, function(d) {
-      min.dist <- min(d)
-      haps <- names(d)[d == min.dist]
-      list(haps = haps, min.dist = min.dist)
-    })
+  # get minimum distance of unknowns to other sequences
+  min.dist <- t(sapply(unk.dist, function(mat) apply(mat, 2, min)))
 
-    is.matched <- sapply(ns.matches, function(match.info) {
-      match.info$min.dist == 0 & length(match.info$haps) == 1
-    })
+  # assign sequences if only one of the minimum sequences is 0
+  hap.assign <- apply(min.dist, 1, function(counts) {
+    num.matches <- sum(counts == 0)
+    if(num.matches == 1) colnames(min.dist)[counts == 0] else NA
+  })
 
-    # assign matched haplotypes
-    assigned <- lapply(ns.matches[is.matched],
-                       function(match.info) match.info$haps)
-    assigned <- do.call(c, assigned)
+  # compile vector of haplotype assignments
+  hap.vec <- c(hap.code, hap.assign)[rownames(x)]
 
-    # identify haplotypes that can't be assigned
-    unassigned.df <- lapply(ns.matches[!is.matched], function(match.info) {
-      data.frame(closest.match = paste(match.info$haps, collapse = ", "),
-                 dist = match.info$min.dist)
-    })
-    unassigned.df <- do.call(rbind, unassigned.df)
-    unassigned <- if(is.null(unassigned.df)) {
-      NULL
-    } else {
-      warning("Some sequences could not be unambiguously assigned to a haplotype:",
-              call. = FALSE, immediate. = TRUE)
-      print(unassigned.df)
-      cat("\n")
-      rep(NA, nrow(unassigned.df))
-    }
-    names(unassigned) <- rownames(unassigned.df)
-
-    # create full haplotype assignment vector
-    hap.code <- c(hap.code, assigned, unassigned)[rownames(x)]
-    unassigned.df
+  # create data.frame of unassigned sequences
+  unassigned.df <- if(any(is.na(hap.vec))) {
+    mat <- min.dist[names(hap.vec)[is.na(hap.vec)], , drop = FALSE]
+    df <- do.call(rbind, lapply(1:nrow(mat), function(r) {
+      min.dist <- min(mat[r, ])
+      i <- which(mat[r, ] == min.dist)
+      haps <- paste(colnames(mat)[i], collapse = ", ")
+      data.frame(haplotype = haps, min.substitutions = min.dist)
+    }))
+    rownames(df) <- rownames(mat)
+    df
   } else NULL
 
-  list(haps = hap.code, hap.seqs = hap.seqs, unassigned = unassigned.df)
+  list(haps = hap.vec, hap.seqs = as.DNAbin(hap.seqs), unassigned = unassigned.df)
 }
 
+
+#' @rdname labelHaplotypes
+#' @export
+#'
+labelHaplotypes.list <- function(x, ...) labelHaplotypes(as.DNAbin(x),...)
+
+
+#' @rdname labelHaplotypes
+#' @export
+#'
+labelHaplotypes.character <- function(x, ...) labelHaplotypes(as.DNAbin(x), ...)
+
+
 #' @rdname labelHaplotypes
 #' @export
 #'
@@ -194,13 +218,12 @@ labelHaplotypes.gtypes <- function(x, ...) {
   )
   has.errors <- sapply(new.haps, is.null)
   if(sum(has.errors) > 0) {
-    msg <- paste(names(new.haps)[has.errors], collapse = ", ")
-    msg <- paste("haplotypes could not be assigned for:", msg)
-    stop(msg)
+    has.errors <- paste(names(new.haps)[has.errors], collapse = ", ")
+    stop("haplotypes could not be assigned for: ", has.errors)
   }
 
   # create haplotype data.frame
-  hap.df <- as.data.frame(x, stringsAsFactors = FALSE)
+  hap.df <- as.data.frame(x)
   for(gene in names(new.haps)) {
     old.haps <- hap.df[, gene]
     hap.df[, gene] <- new.haps[[gene]]$haps[old.haps]
