@@ -1,6 +1,5 @@
 #' @title Remove Sequences
-#' @description Remove sequences not used by samples listed in \code{@@loci} 
-#'   slot.
+#' @description Remove sequences not used by samples listed in \code{@@data} slot.
 #'   
 #' @param g a \linkS4class{gtypes} object.
 #' 
@@ -9,12 +8,13 @@
 #' @author Eric Archer \email{eric.archer@@noaa.gov}
 #' 
 #' @importFrom methods new
+#' @importFrom stats na.omit
 #' @export
 #' 
 removeSequences <- function(g) {
   if(is.null(sequences(g))) return(g)
   new.seqs <- lapply(locNames(g), function(x) {
-    haps <- unique(as.character(loci(g)[[x]]))
+    haps <- alleleNames(g)[[x]]
     getSequences(sequences(g), x)[haps]
   })
   names(new.seqs) <- locNames(g)
