@@ -48,7 +48,7 @@ dl.g
 
 ## ------------------------------------------------------------------------
 dl.haps <- labelHaplotypes(dl.g)
-dl.haps$gtype
+dl.haps$gtypes
 
 ## ------------------------------------------------------------------------
 data(dolph.haps)
